<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */


namespace LMS\Libraries\Base;

use LMS\Libraries\Message;
use LMS\Libraries\Injection;
use LMS\Libraries\ServiceLocator;

defined('_JEXEC') or die;

require_once __DIR__.'/../injection/container.php';
require_once __DIR__.'/../message/manager.php';

/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 * @since       3.1
 */
class Controller extends \JControllerLegacy
{
	/**
	 * @var \JApplication
	 */
	protected $application;

	/**
	 * @var \JInput
	 */
	protected $input;

	/**
	 * @var \JEventDispatcher
	 */
	protected $dispatcher;

	/**
	 * @var \LMS\Libraries\Acl
	 */
	protected $acl;

	/**
	 * @var \JLMS_Config
	 */
	protected $config;

	/**
	 * @var \LMS\Libraries\Injection\Container
	 */
	protected $container;

	/**
	 * @var \JSession
	 */
	protected $session;

	/**
	 * @var \JUser
	 */
	protected $user;

	/**
	 * @var \LMS\Libraries\Message\Manager
	 */
	protected $msgManager;


	function __construct($config=array())
	{
		parent::__construct($config);

		$this->dispatcher = ServiceLocator::get('dispatcher');		
		$this->acl = ServiceLocator::get('acl');
		$this->config = ServiceLocator::get('config');
		$this->application = ServiceLocator::get('application');
		$this->input = $this->application->input;
		$this->session = ServiceLocator::get('session');
		$this->user = ServiceLocator::get('user');
		$this->msgManager = ServiceLocator::get('message_manager');

		$this->container = new Injection\Container();
		$this->container['msgManager'] = $this->msgManager;
	}

	/**
	 * Method to load and return a view object. This method first looks in the
	 * current template directory for a match and, failing that, uses a default
	 * set path to load the view class file.
	 *
	 * Note the "name, prefix, type" order of parameters, which differs from the
	 * "name, type, prefix" order used in related public methods.
	 *
	 * @param   string  $name    The name of the view.
	 * @param   string  $prefix  Optional prefix for the view class name.
	 * @param   string  $type    The type of view.
	 * @param   array   $config  Configuration array for the view. Optional.
	 *
	 * @return  mixed  View object on success; null or error result on failure.
	 *
	 * @since   12.2
	 * @note    Replaces _createView.
	 * @throws  \Exception
	 */
	protected function createView($name, $prefix = '', $type = '', $config = array())
	{
		$viewName = preg_replace('/[^A-Z0-9_]/i', '', $name);
		$viewType = preg_replace('/[^A-Z0-9_]/i', '', $type);

		$viewType = !empty($viewType) ? $viewType : $this->input->get('format', 'html');

		$explodedPrefix = explode('\\', $prefix);
		$newPrefix = '';
		array_shift($explodedPrefix);
		foreach($explodedPrefix as $prefixPart){
			$newPrefix .= ucfirst($prefixPart) . '\\';
		}
		
		$viewClass = 'LMS\\' . $newPrefix . ucfirst($viewName);			
		$viewFile = __DIR__ . '/../../' . trim(strtolower(str_replace('\\', '/', $newPrefix)), '/') . 's/' . strtolower($viewName) . '/view.' . $viewType . '.php';		
		
		if(!file_exists($viewFile)){
			return null;
		}
		require_once $viewFile;		
		return new $viewClass($config);
	}

	/**
	 * Method to load and return a model object.
	 *
	 * @param   string  $name    The name of the model.
	 * @param   string  $prefix  Optional model prefix.
	 * @param   array   $config  Configuration array for the model. Optional.
	 *
	 * @return  mixed   Model object on success; otherwise null failure.
	 *
	 * @since   12.2
	 * @note    Replaces _createModel.
	 */
	protected function createModel($name, $prefix = '', $config = array())
	{
		$type = preg_replace('/[^A-Z0-9_]/i', '', $name);

		$explodedPrefix = explode('\\', $prefix);
		$newPrefix = '';
		array_shift($explodedPrefix);
		foreach($explodedPrefix as $prefixPart){
			$newPrefix .= ucfirst($prefixPart) . '\\';
		}

		$modelClass = 'LMS\\' . $newPrefix . ucfirst($type);
		$modelFile =__DIR__ . '/../../' . trim(strtolower(str_replace('\\', '/', $newPrefix)), '/') . 's/' . strtolower($type) . '.php';

		if(!file_exists($modelFile)){
			return null;
		}
		require_once $modelFile;
		
		$config['tool'] = $this->input->get('tool');

		return new $modelClass($config);
	}


	protected function populateModel(\LMS\Libraries\Base\Model $model)
	{
		$limit = $this->input->get('limit', $this->session->get($model->getName() . '.list.limit', 20));
		$start = $this->input->get('start', false);
		$offset = $start !== false ? $start : $this->input->get('limitstart', $this->session->get($model->getName() . '.list.limitstart', 0));
		$filters = $this->input->get('filter', $this->session->get($model->getName() . '.list.filters', array()), 'array');
		$list = $this->input->get('list', $this->session->get($model->getName() . '.list', null), 'array');

		// WORKAROUND: need to set start to 0 in SEF Mode because otherwise pagination link for first page was not generated in the right way
		// it happens because in SEF Model JRoute set start to 0 only if limitstart > 0
		$router = \JApplication::getRouter();
		if($router->getMode() == JROUTER_MODE_SEF){
			$router->setVar('start', '0');
		}

		$defaultOrdering = $model->getState('list.defaultOrdering');
		if($defaultOrdering){
			$orderingParts = explode(' ', $defaultOrdering);
			$model->setState('list.ordering', $orderingParts[0]);
			$model->setState('list.direction', $orderingParts[1]);
		}

		if($list && isset($list['fullordering'])){
			$orderingParts = explode(' ', $list['fullordering']);
			$model->setState('list.ordering', $orderingParts[0]);
			$model->setState('list.direction', $orderingParts[1]);
		}

		$model->setState('list.limit', $limit);
		$model->setState('list.start', $offset);

		$model->setState('list.filters', $filters);

		$this->session->set($model->getName() . '.list.limit', $limit);
		$this->session->set($model->getName() . '.list.limitstart', $offset);
		$this->session->set($model->getName() . '.list.filters', $filters);
		$this->session->set($model->getName() . '.list', $list);

	}
}